# -*- coding: utf-8 -*-
"""
Scrape Website Data | Cannlytics
Copyright © 2021 Cannlytics
Author: Keegan Skeate <keegan@cannlytics.com>
Created: 1/10/2021

License GPLv3+: GNU GPL version 3 or later <https://gnu.org/licenses/gpl.html>
This is free software: you are free to change and redistribute it.
There is NO WARRANTY, to the extent permitted by law.

Resources:
    https://stackoverflow.com/questions/54416896/how-to-scrape-email-and-phone-numbers-from-a-list-of-websites
    https://hackersandslackers.com/scraping-urls-with-beautifulsoup/
"""
import re
import requests
from bs4 import BeautifulSoup


def get_page_metadata(url):
    """Scrape target URL for metadata."""
    headers = {
        'Access-Control-Allow-Origin': '*',
        'Access-Control-Allow-Methods': 'GET',
        'Access-Control-Allow-Headers': 'Content-Type',
        'Access-Control-Max-Age': '3600',
        'User-Agent': 'Mozilla/5.0 (X11; Ubuntu; Linux x86_64; rv:52.0) Gecko/20100101 Firefox/52.0'
    }
    # Handle url's without http beginning
    if not url.startswith('http'):
        url = 'http://' + url
    response = requests.get(url, headers=headers)
    html = BeautifulSoup(response.content, 'html.parser')
    metadata = {
        'description': get_description(html),
        'image_url': get_image(html),
        'favicon': get_favicon(html, url),
        'brand_color': get_theme_color(html),
    }
    return response, html, metadata


def get_description(html):
    """Scrape page description."""
    description = None
    if html.find("meta", property="description"):
        description = html.find("meta", property="description").get('content')
    elif html.find("meta", property="og:description"):
        description = html.find("meta", property="og:description").get('content')
    elif html.find("meta", property="twitter:description"):
        description = html.find("meta", property="twitter:description").get('content')
    elif html.find("p"):
        description = html.find("p").contents
    if isinstance(description, list):
        try:
            description = description[0]
        except IndexError:
            pass
    return description


def get_image(html):
    """Scrape share image."""
    image = None
    if html.find("meta", property="image"):
        image = html.find("meta", property="image").get('content')
    elif html.find("meta", property="og:image"):
        image = html.find("meta", property="og:image").get('content')
    elif html.find("meta", property="twitter:image"):
        image = html.find("meta", property="twitter:image").get('content')
    elif html.find("img", src=True):
        image = html.find_all("img")[0].get('src')
    return image


def get_favicon(html, url):
    """Scrape favicon."""
    if html.find("link", attrs={"rel": "icon"}):
        favicon = html.find("link", attrs={"rel": "icon"}).get('href')
    elif html.find("link", attrs={"rel": "shortcut icon"}):
        favicon = html.find("link", attrs={"rel": "shortcut icon"}).get('href')
    else:
        favicon = f'{url.rstrip("/")}/favicon.ico'
    return favicon


def get_theme_color(html):
    """Scrape brand color."""
    if html.find("meta", property="theme-color"):
        color = html.find("meta", property="theme-color").get('content')
        return color
    return None


def get_phone(html, response):
    """Scrape phone number."""
    try:
        phone = html.select("a[href*=callto]")[0].text
        return phone
    except:
        pass
    try:
        phone = re.findall(r'\(?\b[2-9][0-9]{2}\)?[-][2-9][0-9]{2}[-][0-9]{4}\b', response.text)[0]
        return phone
    except:
        pass
    try:
       phone = re.findall(r'\(?\b[2-9][0-9]{2}\)?[-. ]?[2-9][0-9]{2}[-. ]?[0-9]{4}\b', response.text)[-1]
       return phone
    except:
        print ('Phone number not found')
        phone = ''
        return phone


def get_email(html, response):
    """Get email."""
    try:
        email = re.findall(r'([a-zA-Z0-9._-]+@[a-zA-Z0-9._-]+\.[a-zA-Z0-9_-]+)', response.text)[-1]
        return email
    except:
        pass
    try:
        email = html.select("a[href*=mailto]")[-1].text
    except:
        print ('Email not found')
        email = ''
        return email


def find_lab_address():
    """
    TODO: Tries to find a lab's address from their website, then Google Maps.
    """
    street, city, state, zipcode = None, None, None, None
    return street, city, state, zipcode


def find_lab_linkedin():
    """
    TODO: Tries to find a lab's LinkedIn URL. (Try to find LinkedIn on homepage?)
    """
    return ""


def find_lab_url():
    """
    TODO: Find a lab's website URL. (Google search for name?)
    """
    return ""


def clean_string_columns(df):
    """Clean string columns in a dataframe."""
    for column in df.columns:
        try:
            df[column] = df[column].str.title()
            df[column] = df[column].str.replace('Llc', 'LLC')
            df[column] = df[column].str.replace('L.L.C.', 'LLC')
            df[column] = df[column].str.strip()
        except AttributeError:
            pass
    return df




# -*- coding: utf-8 -*-
"""
Logistics | Cannlytics
Copyright © 2021 Cannlytics
Author: Keegan Skeate <keegan@cannlytics.com>
Created: 1/10/2021

License GPLv3+: GNU GPL version 3 or later <https://gnu.org/licenses/gpl.html>
This is free software: you are free to change and redistribute it.
There is NO WARRANTY, to the extent permitted by law.
"""
import googlemaps
from googlemaps import places
from firebase_admin import firestore
from time import sleep


def get_google_maps_api_key():
    """Get Google Maps API key."""
    db = firestore.client()
    admin = db.collection("admin")
    google = admin.document("google").get()
    google_data = google.to_dict()
    return google_data["google_maps_api_key"]


def geocode_addresses(df):
    """Geocode addresses in a dataframe."""
    api_key = get_google_maps_api_key()
    gmaps = googlemaps.Client(key=api_key)
    for index, item in df.iterrows():
        # FIXME: Handle existing lat and long more elegantly.
        # try:
        #     if item.latitude and item.longitude:
        #         continue
        # except:
        #     pass
        address = f"{item.street}, {item.city}, {item.state} {item.zip}"
        geocode_result = gmaps.geocode(address)
        if geocode_result:
            df.at[index,'formatted_address'] = geocode_result[0]['formatted_address']
            location = geocode_result[0]['geometry']['location']
            print(item.name, '-->', location)
            df.at[index,'latitude'] = location['lat']
            df.at[index,'longitude'] = location['lng']
            for info in geocode_result[0]['address_components']:
                key = info['types'][0]
                if key == 'administrative_area_level_2':
                    df.at[index,'county'] = info['long_name']

        sleep(2) # Prevents spamming Google's servers (necessary?)
    return df


def search_for_address(query, api_key=None, fields=['formatted_address']):
    """Search for the address of a given name."""
    if api_key is None:
        api_key = get_google_maps_api_key()
    gmaps = googlemaps.Client(key=api_key)
    place = places.find_place(gmaps, query, 'textquery', fields=fields)
    return place['candidates']


def get_place_details(query, api_key=None, fields=[]):
    """Get the place details for a given a name."""
    if api_key is None:
        api_key = get_google_maps_api_key()
    if not fields:
        fields = [
            'formatted_address',
            'photo',
            'opening_hours',
            'website',
        ]
    gmaps = googlemaps.Client(key=api_key)
    search = places.find_place(gmaps, query, 'textquery')
    place_id = search['candidates'][0]['place_id']
    place = places.place(gmaps, place_id, fields=fields)
    return place['result']




# -*- coding: utf-8 -*-
"""
Upload Labs | Cannlytics
Copyright © 2021 Cannlytics
Author: Keegan Skeate <keegan@cannlytics.com>
Created: 1/12/2021

License GPLv3+: GNU GPL version 3 or later <https://gnu.org/licenses/gpl.html>
This is free software: you are free to change and redistribute it.
There is NO WARRANTY, to the extent permitted by law.
"""
import os
import pandas as pd
from django.template.defaultfilters import slugify
from firebase_admin import initialize_app, firestore
from uuid import uuid4


# TODO: Set reference to your Firebase service account.
os.environ["GOOGLE_APPLICATION_CREDENTIALS"] = ""

def upload_labs(input_file, sheet_name="All Labs"):
    """Upload lab data points to Firestore."""
    try:
        initialize_app()
    except ValueError:
        pass    
    database = firestore.client()
    collection = database.collection('labs')
    labs = pd.read_excel(
        input_file,
        sheet_name=sheet_name,
        converters={
            'license_expiration_date':str,
            'license_issue_date':str,
        }
    )
    labs = labs.where(pd.notnull(labs), None)
    for index, row in labs.iterrows():
        _id = str(uuid4())
        docs = collection.where(u'license', u'==', row.license).stream()
        for doc in docs:
            _id = doc.id
        values = row.to_dict()
        values["id"] = _id
        values["slug"] = slugify(values["name"])
        collection.document(_id).set(values, merge=True)
        print('Uploaded lab data for:', row["name"])


if __name__ == "__main__":
    
    upload_labs(input_file="./data/All Labs.xlsx", sheet_name="All Labs")




import setuptools

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

setuptools.setup(
    name="cannlytics", # Replace with your own username
    version="0.0.1",
    author="Cannlytics",
    author_email="contact@cannlytics.com",
    description="Cannlytics provides a user-friendly interface to quickly receive samples, perform analyses, collect and review results, and publish certificates of analysis (CoAs). There are also built in logistics, CRM (client relationship management), inventory management, and invoicing tools.",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/cannlytics/cannlytics-engine",
    packages=setuptools.find_packages(),
    license="GPLv3",
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: GNU General Public License v3 (GPLv3)",
    ],
    python_requires='>=3.6',
    include_package_data=True,
    install_requires=["bs4", "django", "firebase_admin", "googlemaps", "pandas"],
)
